#include <IRremote.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

// LCD Display setup
LiquidCrystal_I2C lcd(0x27, 16, 2);

// IR Receiver variables
const int RECV_PIN = 9;
IRrecv irrecv(RECV_PIN);
decode_results results;

// Game and target status
bool activeTarget = false;
bool activeGame = false;

// Time variables
unsigned long delayStart = 0;
bool delayRunning = false;
unsigned long reactionTime = 0;
unsigned long randomTime = 0;

void setup(){
  // Setup pins
  pinMode(2, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(12, INPUT_PULLUP);

  // Enable IR receiver
  irrecv.enableIRIn();
  irrecv.blink13(true);

  lcd.begin();

  // Draw start text on LCD display
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0, 0);
  String text = "Press the button";
  lcd.print(text);
  
  lcd.setCursor(0, 1);
  text = "to start";
  lcd.print(text);
}

void loop(){
  // Set LED status based on booleans
  digitalWrite(2, activeGame ? HIGH : LOW);
  digitalWrite(8, activeTarget ? HIGH : LOW);

  // If game is not active
  if(!activeGame){
    // Read button input
    int buttonState = digitalRead(12);
    // If button pressed
    if(buttonState == 0){
      // Setup variables for next step
      randomTime = random(1000, 5000);
      delayStart = millis();
      activeGame = true;
      delayRunning = true;
    }
  }
  else{ // Game is active
    // If target is not active
    if(!activeTarget){
      // See if random time has been reached
      if (delayRunning && ((millis() - delayStart) >= randomTime)) {
        // Setup variables for next step
        reactionTime = millis();
        delayRunning = false;
        activeTarget = true;
      }
    }
    else{ // Target is active
      // Check IR receiver input
      if (irrecv.decode(&results)){ // IR receiver received input
        // Reset all variables
        activeTarget = false;
        activeGame = false;
        // Count reaction time
        int time2 = (millis() - reactionTime);

        // Display reaction time
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Time = ");
        lcd.print(time2);
        lcd.print("ms");

        // Resume IR receiver
        irrecv.resume(); 
      }
    }
  }
}
